# -*- coding: utf-8 -*-

from tuikit.emitter import Emitter
from tuikit.common import Coords, Size


class Widget(Emitter):

    '''Base class for all widgets.'''

    def __init__(self, width = 10, height = 10):
        #: Widget name is used for logging etc. Not visible anywhere.
        self.name = None

        #: Parent widget.
        self.parent = None

        #: Top widget (same for every widget in one application).
        self.top = None

        # Position inside parent widget. Modified by layout manager.
        self.position = Coords()

        # Actual size. Modified by layout manager.
        self.size = Size(width, height)

        #: Minimal size of widget. Under normal circumstances
        #: widget will never be sized smaller than this.
        #: Tuple (w, h). Both must be integers >= 1.
        self.sizemin = Size(1,1)

        #: Maximum size of widget. Widget will never be sized bigger than this.
        #: Tuple (w, h). Integers >= 1 or None (meaning no maximum size or infinite).
        self.sizemax = Size(None, None)

        #: Size request. This is default size of the widget. Will be fulfilled if possible.
        #: Tuple (w, h). Integers >= 1 or None (meaning use minumal size).
        self.sizereq = Size(10,10)

        #: When false, the widget is not considered in layout.
        self.allowlayout = True

        #: Allow keyboard focus for this widget.
        self.allow_focus = False

        #: Dictionary containing optional parameters for layout managers etc.
        self.hints = {}

        #: Hidden widget does not affect layout.
        self.hidden = False

        # cursor
        self.cursor = None

        # redraw request
        self._redraw = True

        # event handlers
        self.add_events(
            'resize',
            'draw',
            'focus',
            'unfocus',
            'keypress',
            'mousedown',
            'mouseup',
            'mousemove',
            'mousewheel')


    @property
    def x(self):
        return self.position.x

    @x.setter
    def x(self, value):
        self.position.x = value

    @property
    def y(self):
        return self.position.y

    @y.setter
    def y(self, value):
        self.position.y = value


    @property
    def width(self):
        return self.size.w

    @width.setter
    def width(self, value):
        self.size.w = value
        self.emit('resize')

    @property
    def height(self):
        return self.size.h

    @height.setter
    def height(self, value):
        self.size.h = value
        self.emit('resize')


    def settop(self, top):
        self.top = top


    ### events

    def redraw(self, parent=True):
        self._redraw = True
        if parent and self.parent:
            self.parent._redraw = True

    def draw(self, driver, x, y):
        """Draw the widget.

        This method should not be overriden by subclasses,
        use on_draw instead.

        """
        if self.hidden:
            return True

        self.emit('draw', driver, x, y)

        if self.hasfocus():
            if self.cursor:
                cx, cy = self.cursor
                driver.showcursor(x + cx, y + cy)
            else:
                driver.hidecursor()

    def on_mousedown(self, ev):
        super().on_mousedown(ev)
        self.setfocus()


    ### focus


    def canfocus(self):
        return not self.hidden and self.allow_focus


    def hasfocus(self):
        if self.parent is None:
            return True
        return (self.parent.hasfocus() \
            and self.parent.focuschild == self)


    def setfocus(self):
        if self.hasfocus() or not self.canfocus():
            return
        if self.parent.focuschild:
            self.parent.focuschild.emit('unfocus', self)
        self.parent.focuschild = self
        self.emit('focus')


    ###

    def hint(self, key):
        if key in self.hints:
            return self.hints[key]

    def enclose(self, x, y):
        if self.hidden:
            return False
        if x < self.x or y < self.y \
        or x >= self.x + self.width or y >= self.y + self.height:
            return False
        return  True


    def screentest(self, y, x):
        sy, sx = self.screenyx()
        if y < sy or x < sx or y >= sy + self.height or x >= sx + self.width:
            return False
        return True


    def screenyx(self):
        if self.parent:
            y,x = self.parent.screenyx()
            return self.y + y, self.x + x
        return self.y, self.x


    def hide(self):
        '''Hide widget. Convenience method.'''
        self.hidden = True
        if self.hasfocus():
            self.parent.focusnext()
        self.redraw()


    def show(self):
        '''Show widget. Convenience method.'''
        self.hidden = False
        self.redraw()

